/* $Id: lcd.c,v 1.6 2005/02/04 10:05:31 dijkstra Exp $ */

/*
 * Copyright (c) 2003-2004 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include "lcd.h"
#include "error.h"

int lcd_fd = 0;
char lcd_framebuffer[LCD_X * LCD_Y];

void
lcd_init(char *serial, int capabilities)
{
    bzero(&lcd_framebuffer, sizeof(lcd_framebuffer));
    debug("initialize %s driver @ %s", driver_name, serial);
    driver_init(serial, capabilities);
}

void
lcd_reboot()
{
    bzero(&lcd_framebuffer, sizeof(lcd_framebuffer));
    driver_reset();
}

void
lcd_flush()
{
    driver_flush();
}

void
lcd_clear()
{
    int i;

    for (i = 0; i <= (LCD_X * LCD_Y); i++)
	lcd_framebuffer[i] = LCD_CHAR_SPACE;
}
void
lcd_printf(int x, int y, char *fmt, ...)
{
    va_list args;
    char buf[LCD_MAXSTR];
    int size, i;

    va_start(args, fmt);
    size = vsnprintf(buf, sizeof(buf), fmt, args);
    va_end(args);

    if (size > LCD_MAXSTR || size < 0) {
	size = LCD_MAXSTR;
    }

    if ((x < 0) || (x >= LCD_X) ||
	(y < 0) || (y >= LCD_Y) ||
	(x + size) > LCD_X ) {
	warning("lcd_printf(%d,%d,'%.32s') breaks lcd screen boundaries",
		x, y, &buf[0]);
    }

    for (i=0; i<size; i++) {
	lcd_set(x+i, y, buf[i]);
    }
}

void
lcd_set(int x, int y, char c)
{
    if ((x < 0) || (x >= LCD_X) ||
	(y < 0) || (y >= LCD_Y))
	warning("lcd_set(%d,%d,'%c') breaks lcd screen boundaries",
	      x, y, c);

    lcd_framebuffer[x + (y * LCD_X)] = c;
}

void
lcd_printfraw(char *fmt, ...)
{
    va_list args;
    char buf[LCD_MAXSTR];
    int size;

    va_start(args, fmt);
    size = vsnprintf(buf, sizeof(buf), fmt, args);
    va_end(args);

    write(lcd_fd, buf, size);
}

void
lcd_vbar(int x, int y, int my, int percentage)
{
    int unit;
    int cell;

    unit = 100 / ((y - my) + 1);

    for (cell = y; cell >= my && percentage >= 0; cell--) {
	if (percentage > unit) {
	    driver_put_block(x, cell);
	} else {
	    driver_put_vchar(x, cell, percentage);
	}
	percentage -= unit;
    }
}

void
lcd_hbar(int x, int y, int mx, int percentage)
{
}

void
lcd_printunit(int x, int y, int msp, int lsp, double value)
{
    char buf[LCD_MAXSTR];
    char *units = " KMGTP";
    int i;

    snprintf(&buf[0], sizeof(buf), "%.0f", value);
    for (i = 0; i < strlen(units) && strlen(buf) > msp; i++) {
	value /= 1000;
	snprintf(&buf[0], sizeof(buf), "%.0f", value);
    }

    snprintf(&buf[0], sizeof(buf), "%%%d.%df%c", msp, lsp, units[i]);
    lcd_printf(x, y, buf, value);
}
